@@ -23,26 +23,26 @@ def __main__():
         sys.exit(1)
 
     with open(infile) as inp:
-        with open(outfile, 'w') as out:
-            sequence = ''
+        with open(outfile, "w") as out:
+            sequence = ""
             for line in inp:
-                line = line.rstrip('\r\n')
-                if line.startswith('>'):
+                line = line.rstrip("\r\n")
+                if line.startswith(">"):
                     if sequence:
                         # Flush sequence from previous FASTA record,
                         # removing any white space
-                        out.write("".join(sequence.split()) + '\n')
-                        sequence = ''
+                        out.write("".join(sequence.split()) + "\n")
+                        sequence = ""
                     # Strip off the leading '>' and remove any pre-existing
                     # tabs which would trigger extra columns; write with
                     # tab to separate this from the sequence column:
-                    out.write(line[1:].replace('\t', ' ') + '\t')
+                    out.write(line[1:].replace("\t", " ") + "\t")
                 else:
                     # Continuing sequence,
                     sequence += line
             # End of FASTA file, flush last sequence
             if sequence:
-                out.write("".join(sequence.split()) + '\n')
+                out.write("".join(sequence.split()) + "\n")
 
 
 if __name__ == "__main__":
