@@ -11,8 +11,12 @@ import fileinput
 import sys
 
 from bx.interval_index_file import Indexes
+from bx.intervals.io import GenomicInterval
 
-from galaxy.datatypes.util.gff_util import convert_gff_coords_to_bed, GenomicInterval, GFFReaderWrapper
+from galaxy.datatypes.util.gff_util import (
+    convert_gff_coords_to_bed,
+    GFFReaderWrapper,
+)
 
 
 def main():
