@@ -4,7 +4,7 @@
 import sys
 
 assert sys.version_info[:2] >= (2, 6)
-HEADER_STARTS_WITH = ('@')
+HEADER_STARTS_WITH = "@"
 
 
 def __main__():
@@ -14,16 +14,16 @@ def __main__():
     first_skipped_line = 0
     header_lines = 0
     i = 0
-    with open(input_name) as fh, open(output_name, 'w') as out:
+    with open(input_name) as fh, open(output_name, "w") as out:
         for i, line in enumerate(fh):
-            line = line.rstrip('\r\n')
+            line = line.rstrip("\r\n")
             if line:
                 if line.startswith(HEADER_STARTS_WITH):
                     header_lines += 1
                 else:
                     try:
-                        elems = line.split('\t')
-                        out.write(f'{elems[0]}\t{int(elems[1]) - 1}\t{elems[2]}\t{elems[4]}\t0\t{elems[3]}\n')
+                        elems = line.split("\t")
+                        out.write(f"{elems[0]}\t{int(elems[1]) - 1}\t{elems[2]}\t{elems[4]}\t0\t{elems[3]}\n")
                     except Exception as e:
                         print(e)
                         skipped_lines += 1
@@ -35,7 +35,10 @@ def __main__():
                     first_skipped_line = i + 1
     info_msg = "%i lines converted to BED.  " % (i + 1 - skipped_lines)
     if skipped_lines > 0:
-        info_msg += "Skipped %d blank/comment/invalid lines starting with line #%d." % (skipped_lines, first_skipped_line)
+        info_msg += "Skipped %d blank/comment/invalid lines starting with line #%d." % (
+            skipped_lines,
+            first_skipped_line,
+        )
     print(info_msg)
 
 
