@@ -8,8 +8,8 @@ class InvalidDataProviderSource(TypeError):
     Raised when a unusable source is passed to a provider.
     """
 
-    def __init__(self, source=None, msg=''):
-        msg = msg or f'Invalid source for provider: {source}'
+    def __init__(self, source=None, msg=""):
+        msg = msg or f"Invalid source for provider: {source}"
         super().__init__(msg)
 
 
@@ -27,7 +27,7 @@ class NoProviderAvailable(TypeError):
     Meant to be used within a class that builds dataproviders (e.g. a Datatype)
     """
 
-    def __init__(self, factory_source, format_requested=None, msg=''):
+    def __init__(self, factory_source, format_requested=None, msg=""):
         self.factory_source = factory_source
         self.format_requested = format_requested
         msg = msg or f'No provider available in factory_source "{str(factory_source)}" for format requested'
