@@ -1,58 +1,118 @@
 """
 Proteomics Datatypes
 """
+
 import logging
 import re
+from typing import (
+    IO,
+    List,
+    Optional,
+)
 
 from galaxy.datatypes import data
 from galaxy.datatypes.binary import Binary
 from galaxy.datatypes.data import Text
+from galaxy.datatypes.protocols import (
+    DatasetProtocol,
+    HasExtraFilesAndMetadata,
+)
 from galaxy.datatypes.sequence import Sequence
 from galaxy.datatypes.sniff import (
     build_sniff_from_prefix,
     FilePrefix,
 )
-from galaxy.datatypes.tabular import Tabular, TabularData
+from galaxy.datatypes.tabular import (
+    Tabular,
+    TabularData,
+)
 from galaxy.datatypes.xml import GenericXml
 from galaxy.util import nice_size
 
-
 log = logging.getLogger(__name__)
 
 
 class Wiff(Binary):
     """Class for wiff files."""
+
     edam_data = "data_2536"
     edam_format = "format_3710"
-    file_ext = 'wiff'
-    composite_type = 'auto_primary_file'
+    file_ext = "wiff"
+    composite_type = "auto_primary_file"
 
     def __init__(self, **kwd):
         super().__init__(**kwd)
 
         self.add_composite_file(
-            'wiff',
-            description='AB SCIEX files in .wiff format. This can contain all needed information or only metadata.',
-            is_binary=True)
+            "wiff",
+            description="AB SCIEX files in .wiff format. This can contain all needed information or only metadata.",
+            is_binary=True,
+        )
 
         self.add_composite_file(
-            'wiff_scan',
-            description='AB SCIEX spectra file (wiff.scan), if the corresponding .wiff file only contains metadata.',
-            optional='True', is_binary=True)
+            "wiff_scan",
+            description="AB SCIEX spectra file (wiff.scan), if the corresponding .wiff file only contains metadata.",
+            optional="True",
+            is_binary=True,
+        )
 
-    def generate_primary_file(self, dataset=None):
-        rval = ['<html><head><title>Wiff Composite Dataset </title></head><p/>']
-        rval.append('<div>This composite dataset is composed of the following files:<p/><ul>')
+    def generate_primary_file(self, dataset: HasExtraFilesAndMetadata) -> str:
+        rval = ["<html><head><title>Wiff Composite Dataset </title></head><p/>"]
+        rval.append("<div>This composite dataset is composed of the following files:<p/><ul>")
         for composite_name, composite_file in self.get_composite_files(dataset=dataset).items():
             fn = composite_name
-            opt_text = ''
+            opt_text = ""
             if composite_file.optional:
-                opt_text = ' (optional)'
-            if composite_file.get('description'):
-                rval.append(f"<li><a href=\"{fn}\" type=\"text/plain\">{fn} ({composite_file.get('description')})</a>{opt_text}</li>")
+                opt_text = " (optional)"
+            if composite_file.get("description"):
+                rval.append(
+                    f"<li><a href=\"{fn}\" type=\"text/plain\">{fn} ({composite_file.get('description')})</a>{opt_text}</li>"
+                )
             else:
                 rval.append(f'<li><a href="{fn}" type="text/plain">{fn}</a>{opt_text}</li>')
-        rval.append('</ul></div></html>')
+        rval.append("</ul></div></html>")
+        return "\n".join(rval)
+
+
+class Wiff2(Binary):
+    """Class for wiff2 files."""
+
+    edam_data = "data_2536"
+    edam_format = "format_3710"
+    file_ext = "wiff2"
+    composite_type = "auto_primary_file"
+
+    def __init__(self, **kwd):
+        super().__init__(**kwd)
+
+        self.add_composite_file(
+            "wiff2",
+            description="AB SCIEX files in .wiff2 format. This can contain all needed information or only metadata.",
+            is_binary=True,
+        )
+
+        self.add_composite_file(
+            "wiff_scan",
+            description="AB SCIEX spectra file (wiff.scan), if the corresponding .wiff2 file only contains metadata.",
+            optional="True",
+            is_binary=True,
+        )
+
+    def generate_primary_file(self, dataset: HasExtraFilesAndMetadata) -> str:
+        rval = ["<html><head><title>Wiff2 Composite Dataset </title></head><p/>"]
+        rval.append("<div>This composite dataset is composed of the following files:<p/><ul>")
+        for composite_name, composite_file in self.get_composite_files(dataset=dataset).items():
+            fn = composite_name
+            opt_text = ""
+            if composite_file.optional:
+                opt_text = " (optional)"
+            if composite_file.get("description"):
+                rval.append(
+                    f"<li><a href=\"{fn}\" type=\"text/plain\">{fn} ({composite_file.get('description')})</a>{opt_text}</li>"
+                )
+            else:
+                rval.append(f'<li><a href="{fn}" type="text/plain">{fn}</a>{opt_text}</li>')
+        rval.append("</ul></div></html>")
         return "\n".join(rval)
 
 
@@ -69,31 +129,34 @@ class MzTab(Text):
     >>> MzTab().sniff(fname)
     False
     """
+
     edam_data = "data_3681"
     file_ext = "mztab"
     # section names (except MTD)
     _sections = ["PRH", "PRT", "PEH", "PEP", "PSH", "PSM", "SMH", "SML", "COM"]
     # mandatory metadata fields and list of allowed entries (in lower case)
     # (or None if everything is allowed)
-    _man_mtd = {"mzTab-mode": ["complete", "summary"],
-                "mzTab-type": ['quantification', 'identification'],
-                "description": None}
+    _man_mtd = {
+        "mzTab-mode": ["complete", "summary"],
+        "mzTab-type": ["quantification", "identification"],
+        "description": None,
+    }
     _version_re = r"(1)(\.[0-9])?(\.[0-9])?"
 
     def __init__(self, **kwd):
         super().__init__(**kwd)
 
-    def set_peek(self, dataset):
+    def set_peek(self, dataset: DatasetProtocol, **kwd) -> None:
         """Set the peek and blurb text"""
         if not dataset.dataset.purged:
-            dataset.peek = data.get_file_peek(dataset.file_name)
-            dataset.blurb = 'mzTab Format'
+            dataset.peek = data.get_file_peek(dataset.get_file_name())
+            dataset.blurb = "mzTab Format"
         else:
-            dataset.peek = 'file does not exist'
-            dataset.blurb = 'file purged from disk'
+            dataset.peek = "file does not exist"
+            dataset.blurb = "file purged from disk"
 
-    def sniff_prefix(self, file_prefix: FilePrefix):
-        """ Determines whether the file is the correct type. """
+    def sniff_prefix(self, file_prefix: FilePrefix) -> bool:
+        """Determines whether the file is the correct type."""
         has_version = False
         found_man_mtd = set()
         contents = file_prefix.string_io()
@@ -108,7 +171,7 @@ class MzTab(Text):
                     mandatory_field = self._man_mtd[columns[1]]
                     if mandatory_field is None or columns[2].lower() in mandatory_field:
                         found_man_mtd.add(columns[1])
-            elif not columns[0] in self._sections:
+            elif columns[0] not in self._sections:
                 return False
         return has_version and found_man_mtd == set(self._man_mtd.keys())
 
@@ -125,6 +188,7 @@ class MzTab2(MzTab):
     >>> MzTab2().sniff(fname)
     False
     """
+
     file_ext = "mztab2"
     _sections = ["SMH", "SML", "SFH", "SMF", "SEH", "SME", "COM"]
     _version_re = r"(2)(\.[0-9])?(\.[0-9])?-M$"
@@ -133,14 +197,14 @@ class MzTab2(MzTab):
     def __init__(self, **kwd):
         super().__init__(**kwd)
 
-    def set_peek(self, dataset):
+    def set_peek(self, dataset: DatasetProtocol, **kwd) -> None:
         """Set the peek and blurb text"""
         if not dataset.dataset.purged:
-            dataset.peek = data.get_file_peek(dataset.file_name)
-            dataset.blurb = 'mzTab2 Format'
+            dataset.peek = data.get_file_peek(dataset.get_file_name())
+            dataset.blurb = "mzTab2 Format"
         else:
-            dataset.peek = 'file does not exist'
-            dataset.blurb = 'file purged from disk'
+            dataset.peek = "file does not exist"
+            dataset.blurb = "file purged from disk"
 
 
 @build_sniff_from_prefix
@@ -156,17 +220,33 @@ class Kroenik(Tabular):
     >>> Kroenik().sniff(fname)
     False
     """
+
     file_ext = "kroenik"
 
     def __init__(self, **kwd):
         super().__init__(**kwd)
-        self.column_names = ["File", "First Scan", "Last Scan", "Num of Scans", "Charge", "Monoisotopic Mass", "Base Isotope Peak", "Best Intensity", "Summed Intensity", "First RTime", "Last RTime", "Best RTime", "Best Correlation", "Modifications"]
+        self.column_names = [
+            "File",
+            "First Scan",
+            "Last Scan",
+            "Num of Scans",
+            "Charge",
+            "Monoisotopic Mass",
+            "Base Isotope Peak",
+            "Best Intensity",
+            "Summed Intensity",
+            "First RTime",
+            "Last RTime",
+            "Best RTime",
+            "Best Correlation",
+            "Modifications",
+        ]
 
-    def display_peek(self, dataset):
+    def display_peek(self, dataset: DatasetProtocol) -> str:
         """Returns formated html of peek"""
         return self.make_html_table(dataset, column_names=self.column_names)
 
-    def sniff_prefix(self, file_prefix: FilePrefix):
+    def sniff_prefix(self, file_prefix: FilePrefix) -> bool:
         fh = file_prefix.string_io()
         line = [_.strip() for _ in fh.readline().split("\t")]
         if line != self.column_names:
@@ -194,17 +274,18 @@ class PepList(Tabular):
     >>> PepList().sniff(fname)
     False
     """
+
     file_ext = "peplist"
 
     def __init__(self, **kwd):
         super().__init__(**kwd)
         self.column_names = ["m/z", "rt(min)", "snr", "charge", "intensity"]
 
-    def display_peek(self, dataset):
+    def display_peek(self, dataset: DatasetProtocol) -> str:
         """Returns formated html of peek"""
         return self.make_html_table(dataset, column_names=self.column_names)
 
-    def sniff_prefix(self, file_prefix: FilePrefix):
+    def sniff_prefix(self, file_prefix: FilePrefix) -> bool:
         fh = file_prefix.string_io()
         line = [_.strip() for _ in fh.readline().split("\t")]
         if line == self.column_names:
@@ -230,17 +311,18 @@ class PSMS(Tabular):
     >>> PSMS().sniff(fname)
     False
     """
+
     file_ext = "psms"
 
     def __init__(self, **kwd):
         super().__init__(**kwd)
         self.column_names = ["PSMId", "score", "q-value", "posterior_error_prob", "peptide", "proteinIds"]
 
-    def display_peek(self, dataset):
+    def display_peek(self, dataset: DatasetProtocol) -> str:
         """Returns formated html of peek"""
         return self.make_html_table(dataset, column_names=self.column_names)
 
-    def sniff_prefix(self, file_prefix: FilePrefix):
+    def sniff_prefix(self, file_prefix: FilePrefix) -> bool:
         fh = file_prefix.string_io()
         line = [_.strip() for _ in fh.readline().split("\t")]
         if line == self.column_names:
@@ -254,9 +336,10 @@ class PEFF(Sequence):
     PSI Extended FASTA Format
     https://github.com/HUPO-PSI/PEFF
     """
+
     file_ext = "peff"
 
-    def sniff_prefix(self, file_prefix: FilePrefix):
+    def sniff_prefix(self, file_prefix: FilePrefix) -> bool:
         """
         >>> from galaxy.datatypes.sniff import get_test_fname
         >>> fname = get_test_fname( 'test.peff' )
@@ -275,20 +358,34 @@ class PEFF(Sequence):
 
 class PepXmlReport(Tabular):
     """pepxml converted to tabular report"""
+
     edam_data = "data_2536"
     file_ext = "pepxml.tsv"
 
     def __init__(self, **kwd):
         super().__init__(**kwd)
-        self.column_names = ['Protein', 'Peptide', 'Assumed Charge', 'Neutral Pep Mass (calculated)', 'Neutral Mass', 'Retention Time', 'Start Scan', 'End Scan', 'Search Engine', 'PeptideProphet Probability', 'Interprophet Probability']
+        self.column_names = [
+            "Protein",
+            "Peptide",
+            "Assumed Charge",
+            "Neutral Pep Mass (calculated)",
+            "Neutral Mass",
+            "Retention Time",
+            "Start Scan",
+            "End Scan",
+            "Search Engine",
+            "PeptideProphet Probability",
+            "Interprophet Probability",
+        ]
 
-    def display_peek(self, dataset):
+    def display_peek(self, dataset: DatasetProtocol) -> str:
         """Returns formated html of peek"""
         return self.make_html_table(dataset, column_names=self.column_names)
 
 
 class ProtXmlReport(Tabular):
     """protxml converted to tabular report"""
+
     edam_data = "data_2536"
     file_ext = "protxml.tsv"
     comment_lines = 1
@@ -296,18 +393,33 @@ class ProtXmlReport(Tabular):
     def __init__(self, **kwd):
         super().__init__(**kwd)
         self.column_names = [
-            "Entry Number", "Group Probability",
-            "Protein", "Protein Link", "Protein Probability",
-            "Percent Coverage", "Number of Unique Peptides",
-            "Total Independent Spectra", "Percent Share of Spectrum ID's",
-            "Description", "Protein Molecular Weight", "Protein Length",
-            "Is Nondegenerate Evidence", "Weight", "Precursor Ion Charge",
-            "Peptide sequence", "Peptide Link", "NSP Adjusted Probability",
-            "Initial Probability", "Number of Total Termini",
-            "Number of Sibling Peptides Bin", "Number of Instances",
-            "Peptide Group Designator", "Is Evidence?"]
+            "Entry Number",
+            "Group Probability",
+            "Protein",
+            "Protein Link",
+            "Protein Probability",
+            "Percent Coverage",
+            "Number of Unique Peptides",
+            "Total Independent Spectra",
+            "Percent Share of Spectrum ID's",
+            "Description",
+            "Protein Molecular Weight",
+            "Protein Length",
+            "Is Nondegenerate Evidence",
+            "Weight",
+            "Precursor Ion Charge",
+            "Peptide sequence",
+            "Peptide Link",
+            "NSP Adjusted Probability",
+            "Initial Probability",
+            "Number of Total Termini",
+            "Number of Sibling Peptides Bin",
+            "Number of Instances",
+            "Peptide Group Designator",
+            "Is Evidence?",
+        ]
 
-    def display_peek(self, dataset):
+    def display_peek(self, dataset: DatasetProtocol) -> str:
         """Returns formated html of peek"""
         return self.make_html_table(dataset, column_names=self.column_names)
 
@@ -318,15 +430,16 @@ class Dta(TabularData):
     peptide charge state separated by a space. Subsequent lines contain space
     separated pairs of fragment ion m/z and intensity values.
     """
+
     file_ext = "dta"
     comment_lines = 0
 
-    def set_meta(self, dataset, **kwd):
+    def set_meta(self, dataset: DatasetProtocol, overwrite: bool = True, **kwd) -> None:
         column_types = []
-        data_row = []
+        data_row: List = []
         data_lines = 0
         if dataset.has_data():
-            with open(dataset.file_name) as dtafile:
+            with open(dataset.get_file_name()) as dtafile:
                 for _ in dtafile:
                     data_lines += 1
 
@@ -337,9 +450,9 @@ class Dta(TabularData):
         # Set metadata
         dataset.metadata.data_lines = data_lines
         dataset.metadata.comment_lines = 0
-        dataset.metadata.column_types = ['float', 'float']
+        dataset.metadata.column_types = ["float", "float"]
         dataset.metadata.columns = 2
-        dataset.metadata.column_names = ['m/z', 'intensity']
+        dataset.metadata.column_names = ["m/z", "intensity"]
         dataset.metadata.delimiter = " "
 
 
@@ -365,26 +478,27 @@ class Dta2d(TabularData):
     >>> Dta2d().sniff(fname)
     False
     """
+
     file_ext = "dta2d"
     comment_lines = 0
 
-    def _parse_header(self, line):
+    def _parse_header(self, line: List) -> Optional[List]:
         if len(line) != 3 or len(line[0]) < 3 or not line[0].startswith("#"):
             return None
         line[0] = line[0].lstrip("#")
         line = [_.strip() for _ in line]
-        if 'MZ' not in line or 'INT' not in line or ('MIN' not in line and 'SEC' not in line):
+        if "MZ" not in line or "INT" not in line or ("MIN" not in line and "SEC" not in line):
             return None
         return line
 
-    def _parse_delimiter(self, line):
+    def _parse_delimiter(self, line: str) -> Optional[str]:
         if len(line.split(" ")) == 3:
             return " "
         elif len(line.split("\t")) == 3:
             return "\t"
         return None
 
-    def _parse_dataline(self, line):
+    def _parse_dataline(self, line: List) -> bool:
         try:
             line = [float(_) for _ in line]
         except ValueError:
@@ -393,11 +507,11 @@ class Dta2d(TabularData):
             return False
         return True
 
-    def set_meta(self, dataset, **kwd):
+    def set_meta(self, dataset: DatasetProtocol, overwrite: bool = True, **kwd) -> None:
         data_lines = 0
         delim = None
         if dataset.has_data():
-            with open(dataset.file_name) as dtafile:
+            with open(dataset.get_file_name()) as dtafile:
                 for line in dtafile:
                     if delim is None:
                         delim = self._parse_delimiter(line)
@@ -410,14 +524,14 @@ class Dta2d(TabularData):
 
         dataset.metadata.data_lines = data_lines
         dataset.metadata.comment_lines = 0
-        dataset.metadata.column_types = ['float', 'float', 'float']
+        dataset.metadata.column_types = ["float", "float", "float"]
         dataset.metadata.columns = 3
         if dataset.metadata.column_names is None or dataset.metadata.column_names == []:
             dataset.metadata.comment_lines += 1
             dataset.metadata.data_lines -= 1
-            dataset.metadata.column_names = ['SEC', 'MZ', 'INT']
+            dataset.metadata.column_names = ["SEC", "MZ", "INT"]
 
-    def sniff_prefix(self, file_prefix: FilePrefix):
+    def sniff_prefix(self, file_prefix: FilePrefix) -> bool:
         sep = None
         header = None
         for idx, line in enumerate(file_prefix.line_iterator()):
@@ -466,10 +580,11 @@ class Edta(TabularData):
     >>> Edta().sniff(fname)
     False
     """
+
     file_ext = "edta"
     comment_lines = 0
 
-    def _parse_delimiter(self, line):
+    def _parse_delimiter(self, line: str) -> Optional[str]:
         if len(line.split(" ")) >= 3:
             return " "
         elif len(line.split("\t")) >= 3:
@@ -478,7 +593,7 @@ class Edta(TabularData):
             return "\t"
         return None
 
-    def _parse_type(self, line):
+    def _parse_type(self, line: List) -> Optional[int]:
         """
         parse the type from the header line
         types 1-3 as in the class docs, 0: type 1 wo/wrong header
@@ -498,7 +613,7 @@ class Edta(TabularData):
         else:
             return 3
 
-    def _parse_dataline(self, line, tpe):
+    def _parse_dataline(self, line: List, tpe: Optional[int]) -> bool:
         if tpe == 2 or tpe == 3:
             idx = 4
         else:
@@ -511,7 +626,7 @@ class Edta(TabularData):
             return False
         return True
 
-    def _clean_header(self, line):
+    def _clean_header(self, line: List) -> List:
         for idx, el in enumerate(line):
             el = el.lower()
             if el.startswith("rt"):
@@ -528,12 +643,12 @@ class Edta(TabularData):
                 line[idx] += str(idx // 4)
         return line
 
-    def set_meta(self, dataset, **kwd):
+    def set_meta(self, dataset: DatasetProtocol, overwrite: bool = True, **kwd) -> None:
         data_lines = 0
         delim = None
         tpe = None
         if dataset.has_data():
-            with open(dataset.file_name) as dtafile:
+            with open(dataset.get_file_name()) as dtafile:
                 for idx, line in enumerate(dtafile):
                     if idx == 0:
                         delim = self._parse_delimiter(line)
@@ -560,7 +675,7 @@ class Edta(TabularData):
             dataset.metadata.comment_lines += 1
             dataset.metadata.data_lines -= 1
 
-    def sniff_prefix(self, file_prefix: FilePrefix):
+    def sniff_prefix(self, file_prefix: FilePrefix) -> bool:
         sep = None
         tpe = None
         for idx, line in enumerate(file_prefix.line_iterator()):
@@ -585,34 +700,36 @@ class Edta(TabularData):
 
 
 class ProteomicsXml(GenericXml):
-    """ An enhanced XML datatype used to reuse code across several
-    proteomic/mass-spec datatypes. """
+    """An enhanced XML datatype used to reuse code across several
+    proteomic/mass-spec datatypes."""
+
     edam_data = "data_2536"
     edam_format = "format_2032"
     root: str
 
-    def sniff_prefix(self, file_prefix: FilePrefix):
-        """ Determines whether the file is the correct XML type. """
+    def sniff_prefix(self, file_prefix: FilePrefix) -> bool:
+        """Determines whether the file is the correct XML type."""
         for line in file_prefix.line_iterator():
             line = line.strip()
-            if not line.startswith('<?'):
+            if not line.startswith("<?"):
                 break
         # pattern match <root or <ns:root for any ns string
-        pattern = r'<(\w*:)?%s' % self.root
+        pattern = r"<(\w*:)?%s" % self.root
         return re.search(pattern, line) is not None
 
-    def set_peek(self, dataset):
+    def set_peek(self, dataset: DatasetProtocol, **kwd) -> None:
         """Set the peek and blurb text"""
         if not dataset.dataset.purged:
-            dataset.peek = data.get_file_peek(dataset.file_name)
-            dataset.blurb = self.blurb
+            dataset.peek = data.get_file_peek(dataset.get_file_name())
+            dataset.blurb = "ProteomicsXML data"
         else:
-            dataset.peek = 'file does not exist'
-            dataset.blurb = 'file purged from disk'
+            dataset.peek = "file does not exist"
+            dataset.blurb = "file purged from disk"
 
 
 class ParamXml(ProteomicsXml):
     """store Parameters in XML formal"""
+
     file_ext = "paramxml"
     blurb = "parameters in xmls"
     root = "parameters|PARAMETERS"
@@ -620,14 +737,16 @@ class ParamXml(ProteomicsXml):
 
 class PepXml(ProteomicsXml):
     """pepXML data"""
+
     edam_format = "format_3655"
     file_ext = "pepxml"
-    blurb = 'pepXML data'
+    blurb = "pepXML data"
     root = "msms_pipeline_analysis"
 
 
 class MascotXML(ProteomicsXml):
     """mzXML data"""
+
     file_ext = "mascotxml"
     blurb = "mascot Mass Spectrometry data"
     root = "mascot_search_results"
@@ -635,29 +754,33 @@ class MascotXML(ProteomicsXml):
 
 class MzML(ProteomicsXml):
     """mzML data"""
+
     edam_format = "format_3244"
     file_ext = "mzml"
-    blurb = 'mzML Mass Spectrometry data'
+    blurb = "mzML Mass Spectrometry data"
     root = "(mzML|indexedmzML)"
 
 
 class NmrML(ProteomicsXml):
     """nmrML data"""
+
     # No edam format number yet.
     file_ext = "nmrml"
-    blurb = 'nmrML NMR data'
+    blurb = "nmrML NMR data"
     root = "nmrML"
 
 
 class ProtXML(ProteomicsXml):
     """protXML data"""
+
     file_ext = "protxml"
-    blurb = 'prot XML Search Results'
+    blurb = "prot XML Search Results"
     root = "protein_summary"
 
 
 class MzXML(ProteomicsXml):
     """mzXML data"""
+
     edam_format = "format_3654"
     file_ext = "mzxml"
     blurb = "mzXML Mass Spectrometry data"
@@ -666,6 +789,7 @@ class MzXML(ProteomicsXml):
 
 class MzData(ProteomicsXml):
     """mzData data"""
+
     edam_format = "format_3245"
     file_ext = "mzdata"
     blurb = "mzData Mass Spectrometry data"
@@ -738,8 +862,9 @@ class XquestXML(ProteomicsXml):
 
 class XquestSpecXML(ProteomicsXml):
     """spec.xml"""
+
     file_ext = "spec.xml"
-    blurb = 'xquest_spectra'
+    blurb = "xquest_spectra"
     root = "xquest_spectra"
 
 
@@ -748,27 +873,29 @@ class QCML(ProteomicsXml):
     https://github.com/OpenMS/OpenMS/blob/113c49d01677f7f03343ce7cd542d83c99b351ee/share/OpenMS/SCHEMAS/mzQCML_0_0_5.xsd
     https://github.com/OpenMS/OpenMS/blob/3cfc57ad1788e7ab2bd6dd9862818b2855234c3f/share/OpenMS/SCHEMAS/qcML_0.0.7.xsd
     """
+
     file_ext = "qcml"
-    blurb = 'QualityAssessments to runs'
+    blurb = "QualityAssessments to runs"
     root = "qcML|MzQualityML)"
 
 
 class Mgf(Text):
     """Mascot Generic Format data"""
+
     edam_data = "data_2536"
     edam_format = "format_3651"
     file_ext = "mgf"
 
-    def set_peek(self, dataset):
+    def set_peek(self, dataset: DatasetProtocol, **kwd) -> None:
         """Set the peek and blurb text"""
         if not dataset.dataset.purged:
-            dataset.peek = data.get_file_peek(dataset.file_name)
-            dataset.blurb = 'mgf Mascot Generic Format'
+            dataset.peek = data.get_file_peek(dataset.get_file_name())
+            dataset.blurb = "mgf Mascot Generic Format"
         else:
-            dataset.peek = 'file does not exist'
-            dataset.blurb = 'file purged from disk'
+            dataset.peek = "file does not exist"
+            dataset.blurb = "file purged from disk"
 
-    def sniff(self, filename):
+    def sniff(self, filename: str) -> bool:
         mgf_begin_ions = "BEGIN IONS"
         max_lines = 100
 
@@ -779,24 +906,26 @@ class Mgf(Text):
                     return True
                 if i > max_lines:
                     return False
+        return False
 
 
 class MascotDat(Text):
-    """Mascot search results """
+    """Mascot search results"""
+
     edam_data = "data_2536"
     edam_format = "format_3713"
     file_ext = "mascotdat"
 
-    def set_peek(self, dataset):
+    def set_peek(self, dataset: DatasetProtocol, **kwd) -> None:
         """Set the peek and blurb text"""
         if not dataset.dataset.purged:
-            dataset.peek = data.get_file_peek(dataset.file_name)
-            dataset.blurb = 'mascotdat Mascot Search Results'
+            dataset.peek = data.get_file_peek(dataset.get_file_name())
+            dataset.blurb = "mascotdat Mascot Search Results"
         else:
-            dataset.peek = 'file does not exist'
-            dataset.blurb = 'file purged from disk'
+            dataset.peek = "file does not exist"
+            dataset.blurb = "file purged from disk"
 
-    def sniff(self, filename):
+    def sniff(self, filename: str) -> bool:
         mime_version = "MIME-Version: 1.0 (Generated by Mascot version 1.0)"
         max_lines = 10
 
@@ -807,37 +936,39 @@ class MascotDat(Text):
                     return True
                 if i > max_lines:
                     return False
+        return False
 
 
 class ThermoRAW(Binary):
     """Class describing a Thermo Finnigan binary RAW file"""
+
     edam_data = "data_2536"
     edam_format = "format_3712"
     file_ext = "thermo.raw"
 
-    def sniff(self, filename):
+    def sniff(self, filename: str) -> bool:
         # Thermo Finnigan RAW format is proprietary and hence not well documented.
         # Files start with 2 bytes that seem to differ followed by F\0i\0n\0n\0i\0g\0a\0n
         # This combination represents 17 bytes, but to play safe we read 20 bytes from
         # the start of the file.
         try:
-            header = open(filename, 'rb').read(20)
-            finnigan = b'F\0i\0n\0n\0i\0g\0a\0n'
+            header = open(filename, "rb").read(20)
+            finnigan = b"F\0i\0n\0n\0i\0g\0a\0n"
             if header.find(finnigan) != -1:
                 return True
             return False
         except Exception:
             return False
 
-    def set_peek(self, dataset):
+    def set_peek(self, dataset: DatasetProtocol, **kwd) -> None:
         if not dataset.dataset.purged:
             dataset.peek = "Thermo Finnigan RAW file"
             dataset.blurb = nice_size(dataset.get_size())
         else:
-            dataset.peek = 'file does not exist'
-            dataset.blurb = 'file purged from disk'
+            dataset.peek = "file does not exist"
+            dataset.blurb = "file purged from disk"
 
-    def display_peek(self, dataset):
+    def display_peek(self, dataset: DatasetProtocol) -> str:
         try:
             return dataset.peek
         except Exception:
@@ -846,16 +977,17 @@ class ThermoRAW(Binary):
 
 @build_sniff_from_prefix
 class Msp(Text):
-    """ Output of NIST MS Search Program chemdata.nist.gov/mass-spc/ftp/mass-spc/PepLib.pdf """
+    """Output of NIST MS Search Program chemdata.nist.gov/mass-spc/ftp/mass-spc/PepLib.pdf"""
+
     file_ext = "msp"
 
     @staticmethod
-    def next_line_starts_with(contents, prefix):
+    def next_line_starts_with(contents: IO, prefix: str) -> bool:
         next_line = contents.readline()
         return next_line is not None and next_line.startswith(prefix)
 
-    def sniff_prefix(self, file_prefix: FilePrefix):
-        """ Determines whether the file is a NIST MSP output file."""
+    def sniff_prefix(self, file_prefix: FilePrefix) -> bool:
+        """Determines whether the file is a NIST MSP output file."""
         begin_contents = file_prefix.contents_header
         if "\n" not in begin_contents:
             return False
@@ -866,62 +998,63 @@ class Msp(Text):
 
 
 class SPLibNoIndex(Text):
-    """SPlib without index file """
+    """SPlib without index file"""
+
     file_ext = "splib_noindex"
 
-    def set_peek(self, dataset):
+    def set_peek(self, dataset: DatasetProtocol, **kwd) -> None:
         """Set the peek and blurb text"""
         if not dataset.dataset.purged:
-            dataset.peek = data.get_file_peek(dataset.file_name)
-            dataset.blurb = 'Spectral Library without index files'
+            dataset.peek = data.get_file_peek(dataset.get_file_name())
+            dataset.blurb = "Spectral Library without index files"
         else:
-            dataset.peek = 'file does not exist'
-            dataset.blurb = 'file purged from disk'
+            dataset.peek = "file does not exist"
+            dataset.blurb = "file purged from disk"
 
 
 @build_sniff_from_prefix
 class SPLib(Msp):
     """SpectraST Spectral Library. Closely related to msp format"""
+
     file_ext = "splib"
-    composite_type = 'auto_primary_file'
+    composite_type = "auto_primary_file"
 
     def __init__(self, **kwd):
         super().__init__(**kwd)
-        self.add_composite_file('library.splib',
-                                description='Spectral Library. Contains actual library spectra',
-                                is_binary=False)
-        self.add_composite_file('library.spidx',
-                                description='Spectrum index', is_binary=False)
-        self.add_composite_file('library.pepidx',
-                                description='Peptide index', is_binary=False)
+        self.add_composite_file(
+            "library.splib", description="Spectral Library. Contains actual library spectra", is_binary=False
+        )
+        self.add_composite_file("library.spidx", description="Spectrum index", is_binary=False)
+        self.add_composite_file("library.pepidx", description="Peptide index", is_binary=False)
 
-    def generate_primary_file(self, dataset=None):
-        rval = ['<html><head><title>Spectral Library Composite Dataset </title></head><p/>']
-        rval.append('<div>This composite dataset is composed of the following files:<p/><ul>')
+    def generate_primary_file(self, dataset: HasExtraFilesAndMetadata) -> str:
+        rval = ["<html><head><title>Spectral Library Composite Dataset </title></head><p/>"]
+        rval.append("<div>This composite dataset is composed of the following files:<p/><ul>")
         for composite_name, composite_file in self.get_composite_files(dataset=dataset).items():
             fn = composite_name
-            opt_text = ''
+            opt_text = ""
             if composite_file.optional:
-                opt_text = ' (optional)'
-            if composite_file.get('description'):
-                rval.append(f"<li><a href=\"{fn}\" type=\"text/plain\">{fn} ({composite_file.get('description')})</a>{opt_text}</li>")
+                opt_text = " (optional)"
+            if composite_file.get("description"):
+                rval.append(
+                    f"<li><a href=\"{fn}\" type=\"text/plain\">{fn} ({composite_file.get('description')})</a>{opt_text}</li>"
+                )
             else:
                 rval.append(f'<li><a href="{fn}" type="text/plain">{fn}</a>{opt_text}</li>')
-        rval.append('</ul></div></html>')
+        rval.append("</ul></div></html>")
         return "\n".join(rval)
 
-    def set_peek(self, dataset):
+    def set_peek(self, dataset: DatasetProtocol, **kwd) -> None:
         """Set the peek and blurb text"""
         if not dataset.dataset.purged:
-            dataset.peek = data.get_file_peek(dataset.file_name)
-            dataset.blurb = 'splib Spectral Library Format'
+            dataset.peek = data.get_file_peek(dataset.get_file_name())
+            dataset.blurb = "splib Spectral Library Format"
         else:
-            dataset.peek = 'file does not exist'
-            dataset.blurb = 'file purged from disk'
+            dataset.peek = "file does not exist"
+            dataset.blurb = "file purged from disk"
 
-    def sniff_prefix(self, file_prefix: FilePrefix):
-        """ Determines whether the file is a SpectraST generated file.
-        """
+    def sniff_prefix(self, file_prefix: FilePrefix) -> bool:
+        """Determines whether the file is a SpectraST generated file."""
         contents = file_prefix.string_io()
         return Msp.next_line_starts_with(contents, "Name:") and Msp.next_line_starts_with(contents, "LibID:")
 
@@ -930,21 +1063,21 @@ class SPLib(Msp):
 class Ms2(Text):
     file_ext = "ms2"
 
-    def sniff_prefix(self, file_prefix: FilePrefix):
-        """ Determines whether the file is a valid ms2 file."""
+    def sniff_prefix(self, file_prefix: FilePrefix) -> bool:
+        """Determines whether the file is a valid ms2 file."""
         header_lines = []
         for line in file_prefix.line_iterator():
             if line.strip() == "":
                 continue
-            elif line.startswith('H\t'):
+            elif line.startswith("H\t"):
                 header_lines.append(line)
             else:
                 break
 
-        for header_field in ['CreationDate', 'Extractor', 'ExtractorVersion', 'ExtractorOptions']:
+        for header_field in ["CreationDate", "Extractor", "ExtractorVersion", "ExtractorOptions"]:
             found_header = False
             for header_line in header_lines:
-                if header_line.startswith(f'H\t{header_field}'):
+                if header_line.startswith(f"H\t{header_field}"):
                     found_header = True
                     break
             if not found_header:
@@ -955,46 +1088,45 @@ class Ms2(Text):
 
 # unsniffable binary format, should do something about this
 class XHunterAslFormat(Binary):
-    """ Annotated Spectra in the HLF format http://www.thegpm.org/HUNTER/format_2006_09_15.html """
+    """Annotated Spectra in the HLF format http://www.thegpm.org/HUNTER/format_2006_09_15.html"""
+
     file_ext = "hlf"
 
 
 class Sf3(Binary):
     """Class describing a Scaffold SF3 files"""
+
     file_ext = "sf3"
 
 
 class ImzML(Binary):
     """
-        Class for imzML files.
-        http://www.imzml.org
+    Class for imzML files.
+    http://www.imzml.org
     """
+
     edam_format = "format_3682"
-    file_ext = 'imzml'
-    composite_type = 'auto_primary_file'
+    file_ext = "imzml"
+    composite_type = "auto_primary_file"
 
     def __init__(self, **kwd):
         super().__init__(**kwd)
 
-        self.add_composite_file(
-            'imzml',
-            description='The imzML metadata component.',
-            is_binary=False)
+        self.add_composite_file("imzml", description="The imzML metadata component.", is_binary=False)
 
-        self.add_composite_file(
-            'ibd',
-            description='The mass spectral data component.',
-            is_binary=True)
+        self.add_composite_file("ibd", description="The mass spectral data component.", is_binary=True)
 
-    def generate_primary_file(self, dataset=None):
-        rval = ['<html><head><title>imzML Composite Dataset </title></head><p/>']
-        rval.append('<div>This composite dataset is composed of the following files:<p/><ul>')
+    def generate_primary_file(self, dataset: HasExtraFilesAndMetadata) -> str:
+        rval = ["<html><head><title>imzML Composite Dataset </title></head><p/>"]
+        rval.append("<div>This composite dataset is composed of the following files:<p/><ul>")
         for composite_name, composite_file in self.get_composite_files(dataset=dataset).items():
             fn = composite_name
-            opt_text = ''
-            if composite_file.get('description'):
-                rval.append(f"<li><a href=\"{fn}\" type=\"text/plain\">{fn} ({composite_file.get('description')})</a>{opt_text}</li>")
+            opt_text = ""
+            if composite_file.get("description"):
+                rval.append(
+                    f"<li><a href=\"{fn}\" type=\"text/plain\">{fn} ({composite_file.get('description')})</a>{opt_text}</li>"
+                )
             else:
                 rval.append(f'<li><a href="{fn}" type="text/plain">{fn}</a>{opt_text}</li>')
-        rval.append('</ul></div></html>')
+        rval.append("</ul></div></html>")
         return "\n".join(rval)
