@@ -6,19 +6,26 @@ import copy
 import json
 import logging
 import os
-import shutil
 import sys
 import tempfile
 import weakref
 from collections import OrderedDict
 from collections.abc import Mapping
 from os.path import abspath
-from typing import Any, Iterator, Optional, TYPE_CHECKING, Union
+from typing import (
+    Any,
+    Iterator,
+    Optional,
+    TYPE_CHECKING,
+    Union,
+)
 
+from sqlalchemy import select
 from sqlalchemy.orm import object_session
 from sqlalchemy.orm.attributes import flag_modified
 
 import galaxy.model
+from galaxy.model.base import transaction
 from galaxy.model.scoped_session import galaxy_scoped_session
 from galaxy.security.object_wrapper import sanitize_lists_to_string
 from galaxy.util import (
@@ -62,7 +69,9 @@ class Statement:
     @classmethod
     def process(cls, element):
         for statement, args, kwargs in getattr(element, STATEMENTS, []):
-            statement.target(element, *args, **kwargs)  # statement.target is MetadataElementSpec, element is a Datatype class
+            statement.target(
+                element, *args, **kwargs
+            )  # statement.target is MetadataElementSpec, element is a Datatype class
 
 
 class MetadataCollection(Mapping):
@@ -73,7 +82,11 @@ class MetadataCollection(Mapping):
     retrieved, returning default values in cases when metadata is not set.
     """
 
-    def __init__(self, parent: Union["DatasetInstance", "NoneDataset"], session: Optional[Union[galaxy_scoped_session, 'SessionlessContext']] = None) -> None:
+    def __init__(
+        self,
+        parent: Union["DatasetInstance", "NoneDataset"],
+        session: Optional[Union[galaxy_scoped_session, "SessionlessContext"]] = None,
+    ) -> None:
         self.parent = parent
         self._session = session
         # initialize dict if needed
@@ -90,6 +103,7 @@ class MetadataCollection(Mapping):
         # collection: hda/lda (parent) <--> MetadataCollection (self) ; needs to be
         # hashable, so cannot use proxy.
         self.__dict__["_parent"] = weakref.ref(parent)
+
     parent = property(get_parent, set_parent)
 
     @property
@@ -125,6 +139,7 @@ class MetadataCollection(Mapping):
 
     def __bool__(self):
         return bool(self.parent._metadata)
+
     __nonzero__ = __bool__
 
     def __getattr__(self, name):
@@ -140,14 +155,14 @@ class MetadataCollection(Mapping):
     def __setattr__(self, name, value):
         if name == "parent":
             return self.set_parent(value)
-        elif name == '_session':
+        elif name == "_session":
             super().__setattr__(name, value)
         else:
             if name in self.spec:
                 self.parent._metadata[name] = self.spec[name].unwrap(value)
             else:
                 self.parent._metadata[name] = value
-            flag_modified(self.parent, '_metadata')
+            flag_modified(self.parent, "_metadata")
 
     def remove_key(self, name):
         if name in self.parent._metadata:
@@ -199,11 +214,11 @@ class MetadataCollection(Mapping):
     def from_JSON_dict(self, filename=None, path_rewriter=None, json_dict=None):
         dataset = self.parent
         if filename is not None:
-            log.debug(f'loading metadata from file for: {dataset.__class__.__name__} {dataset.id}')
+            log.debug(f"loading metadata from file for: {dataset.__class__.__name__} {dataset.id}")
             with open(filename) as fh:
                 JSONified_dict = json.load(fh)
         elif json_dict is not None:
-            log.debug(f'loading metadata from dict for: {dataset.__class__.__name__} {dataset.id}')
+            log.debug(f"loading metadata from dict for: {dataset.__class__.__name__} {dataset.id}")
             if isinstance(json_dict, str):
                 JSONified_dict = json.loads(json_dict)
             elif isinstance(json_dict, dict):
@@ -225,7 +240,7 @@ class MetadataCollection(Mapping):
                 external_value = JSONified_dict[name]
                 param = spec.param
                 if isinstance(param, FileParameter):
-                    from_ext_kwds['path_rewriter'] = path_rewriter
+                    from_ext_kwds["path_rewriter"] = path_rewriter
                 value = param.from_external_value(external_value, dataset, **from_ext_kwds)
                 metadata_name_value[name] = value
             elif name in dataset._metadata:
@@ -234,13 +249,13 @@ class MetadataCollection(Mapping):
                 del dataset._metadata[name]
         for name, value in metadata_name_value.items():
             dataset._metadata[name] = value
-        if '__extension__' in JSONified_dict:
-            dataset.extension = JSONified_dict['__extension__']
-        if '__validated_state__' in JSONified_dict:
-            dataset.validated_state = JSONified_dict['__validated_state__']
-        if '__validated_state_message__' in JSONified_dict:
-            dataset.validated_state_message = JSONified_dict['__validated_state_message__']
-        flag_modified(dataset, '_metadata')
+        if "__extension__" in JSONified_dict:
+            dataset.extension = JSONified_dict["__extension__"]
+        if "__validated_state__" in JSONified_dict:
+            dataset.validated_state = JSONified_dict["__validated_state__"]
+        if "__validated_state_message__" in JSONified_dict:
+            dataset.validated_state_message = JSONified_dict["__validated_state_message__"]
+        flag_modified(dataset, "_metadata")
 
     def to_JSON_dict(self, filename=None):
         meta_dict = {}
@@ -248,19 +263,19 @@ class MetadataCollection(Mapping):
         for name, spec in self.spec.items():
             if name in dataset_meta_dict:
                 meta_dict[name] = spec.param.to_external_value(dataset_meta_dict[name])
-        if '__extension__' in dataset_meta_dict:
-            meta_dict['__extension__'] = dataset_meta_dict['__extension__']
-        if '__validated_state__' in dataset_meta_dict:
-            meta_dict['__validated_state__'] = dataset_meta_dict['__validated_state__']
-        if '__validated_state_message__' in dataset_meta_dict:
-            meta_dict['__validated_state_message__'] = dataset_meta_dict['__validated_state_message__']
+        if "__extension__" in dataset_meta_dict:
+            meta_dict["__extension__"] = dataset_meta_dict["__extension__"]
+        if "__validated_state__" in dataset_meta_dict:
+            meta_dict["__validated_state__"] = dataset_meta_dict["__validated_state__"]
+        if "__validated_state_message__" in dataset_meta_dict:
+            meta_dict["__validated_state_message__"] = dataset_meta_dict["__validated_state_message__"]
         try:
             encoded_meta_dict = galaxy.model.custom_types.json_encoder.encode(meta_dict)
         except Exception as e:
             raise Exception(f"Failed encoding metadata dictionary: {meta_dict}") from e
         if filename is None:
             return encoded_meta_dict
-        with open(filename, 'wt+') as fh:
+        with open(filename, "w+") as fh:
             fh.write(encoded_meta_dict)
 
     def __getstate__(self):
@@ -290,7 +305,7 @@ class MetadataSpecCollection(OrderedDict):
 
     def __repr__(self):
         # force elements to draw with __str__ for sphinx-apidoc
-        return ', '.join(item.__str__() for item in self.values())
+        return ", ".join(item.__str__() for item in self.values())
 
 
 class MetadataParameter:
@@ -357,15 +372,26 @@ class MetadataElementSpec:
     is a MetadataSpecCollection) of datatype.
     """
 
-    def __init__(self, datatype, name=None, desc=None,
-                 param=MetadataParameter, default=None, no_value=None,
-                 visible=True, set_in_upload=False, **kwargs):
+    def __init__(
+        self,
+        datatype,
+        name=None,
+        desc=None,
+        param=MetadataParameter,
+        default=None,
+        no_value=None,
+        visible=True,
+        set_in_upload=False,
+        optional=False,
+        **kwargs,
+    ):
         self.name = name
         self.desc = desc or name
         self.default = default
         self.no_value = no_value
         self.visible = visible
         self.set_in_upload = set_in_upload
+        self.optional = optional
         # Catch-all, allows for extra attributes to be set
         self.__dict__.update(kwargs)
         # set up param last, as it uses values set above
@@ -376,7 +402,7 @@ class MetadataElementSpec:
         # (The answer is yes, but not all datatypes control optionality appropriately at this point.)
         # This allows us to check that inherited MetadataElement instances from datatypes that set
         # check_required_metadata have been reviewed and considered really required.
-        self.check_required_metadata = datatype.__dict__.get('check_required_metadata', False)
+        self.check_required_metadata = datatype.__dict__.get("check_required_metadata", False)
 
     def get(self, name, default=None):
         return self.__dict__.get(name, default)
@@ -397,7 +423,7 @@ class MetadataElementSpec:
         # TODO??: assuming param is the class of this MetadataElementSpec - add the plain class name for that
         spec_dict = dict(param_class=self.param.__class__.__name__)
         spec_dict.update(self.__dict__)
-        return ("{name} ({param_class}): {desc}, defaults to '{default}'".format(**spec_dict))
+        return "{name} ({param_class}): {desc}, defaults to '{default}'".format(**spec_dict)
 
 
 # create a statement class that, when called,
@@ -469,19 +495,21 @@ class SelectParameter(MetadataParameter):
 
 
 class DBKeyParameter(SelectParameter):
-
     def get_field(self, value=None, context=None, other_values=None, values=None, **kwd):
         context = context or {}
         other_values = other_values or {}
         try:
-            values = kwd['trans'].app.genome_builds.get_genome_build_names(kwd['trans'])
+            values = kwd["trans"].app.genome_builds.get_genome_build_names(kwd["trans"])
         except KeyError:
             pass
         return super().get_field(value, context, other_values, values, **kwd)
 
+    def make_copy(self, value, target_context: MetadataCollection, source_context):
+        value = listify(value)
+        return super().make_copy(value, target_context, source_context)
+
 
 class RangeParameter(SelectParameter):
-
     def __init__(self, spec):
         SelectParameter.__init__(self, spec)
         # The spec must be set with min and max values
@@ -495,7 +523,9 @@ class RangeParameter(SelectParameter):
 
         if values is None:
             values = list(zip(range(self.min, self.max, self.step), range(self.min, self.max, self.step)))
-        return SelectParameter.get_field(self, value=value, context=context, other_values=other_values, values=values, **kwd)
+        return SelectParameter.get_field(
+            self, value=value, context=context, other_values=other_values, values=values, **kwd
+        )
 
     @classmethod
     def marshal(cls, value):
@@ -505,7 +535,6 @@ class RangeParameter(SelectParameter):
 
 
 class ColumnParameter(RangeParameter):
-
     def get_field(self, value=None, context=None, other_values=None, values=None, **kwd):
         context = context or {}
         other_values = other_values or {}
@@ -513,23 +542,22 @@ class ColumnParameter(RangeParameter):
         if values is None and context:
             column_range = range(1, (context.columns or 0) + 1, 1)
             values = list(zip(column_range, column_range))
-        return RangeParameter.get_field(self, value=value, context=context, other_values=other_values, values=values, **kwd)
+        return RangeParameter.get_field(
+            self, value=value, context=context, other_values=other_values, values=values, **kwd
+        )
 
 
 class ColumnTypesParameter(MetadataParameter):
-
     def to_string(self, value):
         return ",".join(map(str, value))
 
 
 class ListParameter(MetadataParameter):
-
     def to_string(self, value):
         return ",".join(str(x) for x in value)
 
 
 class DictParameter(MetadataParameter):
-
     def to_string(self, value):
         return json.dumps(value)
 
@@ -539,7 +567,6 @@ class DictParameter(MetadataParameter):
 
 
 class PythonObjectParameter(MetadataParameter):
-
     def to_string(self, value):
         if not value:
             return self.spec._to_string(self.spec.no_value)
@@ -556,11 +583,10 @@ class PythonObjectParameter(MetadataParameter):
 
 
 class FileParameter(MetadataParameter):
-
     def to_string(self, value):
         if not value:
             return str(self.spec.no_value)
-        return value.file_name
+        return value.get_file_name()
 
     def to_safe_string(self, value):
         # We do not sanitize file names
@@ -577,14 +603,29 @@ class FileParameter(MetadataParameter):
         if isinstance(value, galaxy.model.MetadataFile) or isinstance(value, MetadataTempFile):
             return value
         if isinstance(value, int):
-            return session.query(galaxy.model.MetadataFile).get(value)
+            return session.get(galaxy.model.MetadataFile, value)
         else:
-            return session.query(galaxy.model.MetadataFile).filter_by(uuid=value).one()
+            wrapped_value = session.execute(
+                select(galaxy.model.MetadataFile).filter_by(uuid=value)
+            ).scalar_one_or_none()
+            if wrapped_value:
+                return wrapped_value
+            else:
+                # If we've simultaneously copied the  dataset and we've changed the datatype on the
+                # copy we may not have committed the MetadataFile yet, so we need to commit the session.
+                # TODO: It would be great if we can avoid the commit in the future.
+                with transaction(session):
+                    session.commit()
+            return session.execute(select(galaxy.model.MetadataFile).filter_by(uuid=value)).scalar_one_or_none()
 
     def make_copy(self, value, target_context: MetadataCollection, source_context):
         session = target_context._object_session(target_context.parent)
         value = self.wrap(value, session=session)
         target_dataset = target_context.parent.dataset
+        if value and not value.id:
+            # This is a new MetadataFile object, we're not copying to another dataset.
+            # Just use it.
+            return self.unwrap(value)
         if value and target_dataset.object_store.exists(target_dataset):
             # Only copy MetadataFile if the target dataset has been created in an object store.
             # All current datatypes re-generate MetadataFile objects when setting metadata,
@@ -592,10 +633,12 @@ class FileParameter(MetadataParameter):
             new_value = galaxy.model.MetadataFile(dataset=target_context.parent, name=self.spec.name)
             session.add(new_value)
             try:
-                shutil.copy(value.file_name, new_value.file_name)
+                new_value.update_from_file(value.get_file_name())
             except AssertionError:
-                session(target_context.parent).flush()
-                shutil.copy(value.file_name, new_value.file_name)
+                tmp_session = session(target_context.parent)
+                with transaction(tmp_session):
+                    tmp_session.commit()
+                new_value.update_from_file(value.get_file_name())
             return self.unwrap(new_value)
         return None
 
@@ -621,17 +664,12 @@ class FileParameter(MetadataParameter):
             if mf is None:
                 mf = self.new_file(dataset=parent, **value.kwds)
             # Ensure the metadata file gets updated with content
-            file_name = value.file_name
+            file_name = value.get_file_name()
             if path_rewriter:
                 # Job may have run with a different (non-local) tmp/working
                 # directory. Correct.
                 file_name = path_rewriter(file_name)
-            parent.dataset.object_store.update_from_file(mf,
-                                                         file_name=file_name,
-                                                         extra_dir='_metadata_files',
-                                                         extra_dir_at_root=True,
-                                                         alt_name=os.path.basename(mf.file_name))
-            os.unlink(file_name)
+            mf.update_from_file(file_name)
             value = mf.id
         return value
 
@@ -645,55 +683,55 @@ class FileParameter(MetadataParameter):
             value = MetadataTempFile.to_JSON(value)
         return value
 
-    def new_file(self, dataset=None, **kwds):
+    def new_file(self, dataset=None, metadata_tmp_files_dir=None, **kwds):
         # If there is a place to store the file (i.e. an object_store has been bound to
         # Dataset) then use a MetadataFile and assume it is accessible. Otherwise use
         # a MetadataTempFile.
-        if getattr(dataset.dataset, "object_store", False):
+        if getattr(dataset.dataset, "object_store", False) and not metadata_tmp_files_dir:
             mf = galaxy.model.MetadataFile(name=self.spec.name, dataset=dataset, **kwds)
             sa_session = object_session(dataset)
             if sa_session:
                 sa_session.add(mf)
-                sa_session.flush()  # flush to assign id
+                with transaction(sa_session):
+                    sa_session.commit()  # commit to assign id
             return mf
         else:
             # we need to make a tmp file that is accessable to the head node,
             # we will be copying its contents into the MetadataFile objects filename after restoring from JSON
             # we do not include 'dataset' in the kwds passed, as from_JSON_value() will handle this for us
-            return MetadataTempFile(**kwds)
+            return MetadataTempFile(metadata_tmp_files_dir=metadata_tmp_files_dir, **kwds)
 
 
 # This class is used when a database file connection is not available
 class MetadataTempFile:
-    tmp_dir = 'database/tmp'  # this should be overwritten as necessary in calling scripts
+    tmp_dir = "database/tmp"  # this should be overwritten as necessary in calling scripts
 
-    def __init__(self, **kwds):
+    def __init__(self, metadata_tmp_files_dir=None, **kwds):
         self.kwds = kwds
+        if metadata_tmp_files_dir:
+            self.tmp_dir = metadata_tmp_files_dir
         self._filename = None
 
-    @property
-    def file_name(self):
+    def get_file_name(self):
         if self._filename is None:
             # we need to create a tmp file, accessable across all nodes/heads, save the name, and return it
             self._filename = abspath(tempfile.NamedTemporaryFile(dir=self.tmp_dir, prefix="metadata_temp_file_").name)
-            open(self._filename, 'wb+')  # create an empty file, so it can't be reused using tempfile
+            open(self._filename, "wb+")  # create an empty file, so it can't be reused using tempfile
         return self._filename
 
     def to_JSON(self):
-        return {'__class__': self.__class__.__name__,
-                'filename': self.file_name,
-                'kwds': self.kwds}
+        return {"__class__": self.__class__.__name__, "filename": self.get_file_name(), "kwds": self.kwds}
 
     @classmethod
     def from_JSON(cls, json_dict):
         # need to ensure our keywords are not unicode
-        rval = cls(**stringify_dictionary_keys(json_dict['kwds']))
-        rval._filename = json_dict['filename']
+        rval = cls(**stringify_dictionary_keys(json_dict["kwds"]))
+        rval._filename = json_dict["filename"]
         return rval
 
     @classmethod
     def is_JSONified_value(cls, value):
-        return (isinstance(value, dict) and value.get('__class__', None) == cls.__name__)
+        return isinstance(value, dict) and value.get("__class__", None) == cls.__name__
 
     @classmethod
     def cleanup_from_JSON_dict_filename(cls, filename):
@@ -702,11 +740,11 @@ class MetadataTempFile:
                 for value in json.load(fh).values():
                     if cls.is_JSONified_value(value):
                         value = cls.from_JSON(value)
-                    if isinstance(value, cls) and os.path.exists(value.file_name):
-                        log.debug('Cleaning up abandoned MetadataTempFile file: %s', value.file_name)
-                        os.unlink(value.file_name)
+                    if isinstance(value, cls) and os.path.exists(value.get_file_name()):
+                        log.debug("Cleaning up abandoned MetadataTempFile file: %s", value.get_file_name())
+                        os.unlink(value.get_file_name())
         except Exception as e:
-            log.debug('Failed to cleanup MetadataTempFile temp files from %s: %s', filename, unicodify(e))
+            log.debug("Failed to cleanup MetadataTempFile temp files from %s: %s", filename, unicodify(e))
 
 
 __all__ = (
