@@ -1,6 +1,12 @@
 import logging
 import re
-from typing import Dict, List, Optional, Tuple
+from typing import (
+    Dict,
+    List,
+    Optional,
+    Tuple,
+    TYPE_CHECKING,
+)
 
 from sqlalchemy.exc import IntegrityError
 from sqlalchemy.orm import sessionmaker
@@ -8,12 +14,21 @@ from sqlalchemy.sql import select
 from sqlalchemy.sql.expression import func
 
 import galaxy.model
+from galaxy.exceptions import ItemOwnershipException
+from galaxy.model.base import transaction
 from galaxy.model.scoped_session import galaxy_scoped_session
 from galaxy.util import (
     strip_control_characters,
     unicodify,
 )
 
+if TYPE_CHECKING:
+    from galaxy.model import (
+        GalaxySession,
+        Tag,
+        User,
+    )
+
 log = logging.getLogger(__name__)
 
 
@@ -30,46 +45,56 @@ class TagHandler:
     Manages CRUD operations related to tagging objects.
     """
 
-    def __init__(self, sa_session: galaxy_scoped_session) -> None:
+    def __init__(self, sa_session: galaxy_scoped_session, galaxy_session=None) -> None:
         self.sa_session = sa_session
         # Minimum tag length.
         self.min_tag_len = 1
         # Maximum tag length.
         self.max_tag_len = 255
         # Tag separator.
-        self.tag_separators = ',;'
+        self.tag_separators = ",;"
         # Hierarchy separator.
-        self.hierarchy_separator = '.'
+        self.hierarchy_separator = "."
         # Key-value separator.
         self.key_value_separators = "=:"
         # Initialize with known classes - add to this in subclasses.
         self.item_tag_assoc_info: Dict[str, ItemTagAssocInfo] = {}
+        # Can't include type annotation in signature, because lagom will attempt to look up
+        # GalaxySession, but can't find it due to the circular import
+        self.galaxy_session: Optional["GalaxySession"] = galaxy_session
 
-    def create_tag_handler_session(self):
+    def create_tag_handler_session(self, galaxy_session: Optional["GalaxySession"]):
         # Creates a transient tag handler that avoids repeated flushes
-        return GalaxyTagHandlerSession(self.sa_session)
+        return GalaxyTagHandlerSession(self.sa_session, galaxy_session=galaxy_session)
 
     def add_tags_from_list(self, user, item, new_tags_list, flush=True):
         new_tags_set = set(new_tags_list)
         if item.tags:
-            new_tags_set.update(self.get_tags_str(item.tags).split(','))
+            new_tags_set.update(self.get_tags_str(item.tags).split(","))
         return self.set_tags_from_list(user, item, new_tags_set, flush=flush)
 
     def remove_tags_from_list(self, user, item, tag_to_remove_list, flush=True):
         tag_to_remove_set = set(tag_to_remove_list)
-        tags_set = {_.strip() for _ in self.get_tags_str(item.tags).split(',')}
+        tags_set = {_.strip() for _ in self.get_tags_str(item.tags).split(",")}
         if item.tags:
             tags_set -= tag_to_remove_set
         return self.set_tags_from_list(user, item, tags_set, flush=flush)
 
-    def set_tags_from_list(self, user, item, new_tags_list, flush=True):
+    def set_tags_from_list(
+        self,
+        user,
+        item,
+        new_tags_list,
+        flush=True,
+    ):
         # precondition: item is already security checked against user
         # precondition: incoming tags is a list of sanitized/formatted strings
         self.delete_item_tags(user, item)
-        new_tags_str = ','.join(new_tags_list)
-        self.apply_item_tags(user, item, unicodify(new_tags_str, 'utf-8'), flush=flush)
+        new_tags_str = ",".join(new_tags_list)
+        self.apply_item_tags(user, item, unicodify(new_tags_str, "utf-8"), flush=flush)
         if flush:
-            self.sa_session.flush()
+            with transaction(self.sa_session):
+                self.sa_session.commit()
         return item.tags
 
     def get_tag_assoc_class(self, item_class):
@@ -88,18 +113,18 @@ class TagHandler:
         if not item_tag_assoc_class:
             return []
         # Build select statement.
-        cols_to_select = [item_tag_assoc_class.table.c.tag_id, func.count('*')]
         from_obj = item_tag_assoc_class.table.join(item_class.table).join(galaxy.model.Tag.table)
-        where_clause = (self.get_id_col_in_item_tag_assoc_table(item_class) == item.id)
-        order_by = [func.count("*").desc()]
+        where_clause = self.get_id_col_in_item_tag_assoc_table(item_class) == item.id
         group_by = item_tag_assoc_class.table.c.tag_id
         # Do query and get result set.
-        query = select(columns=cols_to_select,
-                       from_obj=from_obj,
-                       whereclause=where_clause,
-                       group_by=group_by,
-                       order_by=order_by,
-                       limit=limit)
+        query = (
+            select(item_tag_assoc_class.table.c.tag_id, func.count())
+            .select_from(from_obj)
+            .where(where_clause)
+            .group_by(group_by)
+            .order_by(func.count().desc())
+            .limit(limit)
+        )
         result_set = self.sa_session.execute(query)
         # Return community tags.
         community_tags = []
@@ -109,8 +134,7 @@ class TagHandler:
         return community_tags
 
     def get_tool_tags(self):
-        query = select(columns=[galaxy.model.ToolTagAssociation.table.c.tag_id],
-                       from_obj=galaxy.model.ToolTagAssociation.table).distinct()
+        query = select(galaxy.model.ToolTagAssociation.table.c.tag_id).distinct()
         result_set = self.sa_session.execute(query)
 
         tags = []
@@ -119,20 +143,21 @@ class TagHandler:
             tags.append(self.get_tag_by_id(tag_id))
         return tags
 
-    def remove_item_tag(self, user, item, tag_name):
+    def remove_item_tag(self, user: "User", item, tag_name: str):
         """Remove a tag from an item."""
+        self._ensure_user_owns_item(user, item)
         # Get item tag association.
-        item_tag_assoc = self._get_item_tag_assoc(user, item, tag_name)
         # Remove association.
-        if item_tag_assoc:
+        if item_tag_assoc := self._get_item_tag_assoc(user, item, tag_name):
             # Delete association.
             self.sa_session.delete(item_tag_assoc)
             item.tags.remove(item_tag_assoc)
             return True
         return False
 
-    def delete_item_tags(self, user, item):
+    def delete_item_tags(self, user: Optional["User"], item):
         """Delete tags from an item."""
+        self._ensure_user_owns_item(user, item)
         # Delete item-tag associations.
         for tag in item.tags:
             if tag.id:
@@ -141,6 +166,31 @@ class TagHandler:
         # Delete tags from item.
         del item.tags[:]
 
+    def _ensure_user_owns_item(self, user: Optional["User"], item):
+        """Raises exception if user does not own item.
+        Notice that even admin users cannot directly modify tags on items they do not own.
+        To modify tags on items they don't own, admin users must impersonate the item's owner.
+        """
+        if getattr(item, "id", None) is None:
+            # Item is not persisted, likely it is being copied from an existing, so no need
+            # to check ownership at this point.
+            return
+        # Prefer checking ownership via history (or associated history).
+        # When checking multiple items in batch this should save a few lazy-loads
+        is_owner = False
+        history = item if isinstance(item, galaxy.model.History) else getattr(item, "history", None)
+        if not user:
+            if self.galaxy_session and history:
+                # anon users can only tag histories and history items,
+                # and should only have a single history
+                if history == self.galaxy_session.current_history:
+                    return
+            raise ItemOwnershipException("User does not own item.")
+        user_id = history.user_id if history else getattr(item, "user_id", None)
+        is_owner = user_id == user.id
+        if not is_owner:
+            raise ItemOwnershipException("User does not own item.")
+
     def item_has_tag(self, user, item, tag):
         """Returns true if item is has a given tag."""
         # Get tag name.
@@ -156,7 +206,15 @@ class TagHandler:
             return True
         return False
 
-    def apply_item_tag(self, user, item, name, value=None, flush=True):
+    def apply_item_tag(
+        self,
+        user: Optional["User"],
+        item,
+        name,
+        value=None,
+        flush=True,
+    ):
+        self._ensure_user_owns_item(user, item)
         # Use lowercase name for searching/creating tag.
         if name is None:
             return
@@ -187,39 +245,53 @@ class TagHandler:
         item_tag_assoc.user_value = value
         item_tag_assoc.value = lc_value
         if flush:
-            self.sa_session.flush()
+            with transaction(self.sa_session):
+                self.sa_session.commit()
         return item_tag_assoc
 
-    def apply_item_tags(self, user, item, tags_str, flush=True):
+    def apply_item_tags(
+        self,
+        user: Optional["User"],
+        item,
+        tags_str: Optional[str],
+        flush=True,
+    ):
         """Apply tags to an item."""
+        self._ensure_user_owns_item(user, item)
         # Parse tags.
         parsed_tags = self.parse_tags(tags_str)
         # Apply each tag.
         for name, value in parsed_tags:
             self.apply_item_tag(user, item, name, value, flush=flush)
 
-    def get_tags_str(self, tags):
-        """Build a string from an item's tags."""
-        # Return empty string if there are no tags.
+    def get_tags_list(self, tags) -> List[str]:
+        """Build a list of tags from an item's tags."""
+        # Return empty list if there are no tags.
         if not tags:
-            return ""
-        # Create string of tags.
-        tags_str_list = list()
+            return []
+        # Create list of tags.
+        tags_list: List[str] = []
         for tag in tags:
             tag_str = tag.user_tname
             if tag.value is not None:
                 tag_str += f":{tag.user_value}"
-            tags_str_list.append(tag_str)
-        return ", ".join(tags_str_list)
+            tags_list.append(tag_str)
+        return tags_list
+
+    def get_tags_str(self, tags):
+        """Build a string from an item's tags."""
+        tags_list = self.get_tags_list(tags)
+        # Return empty string if there are no tags.
+        return ", ".join(tags_list)
 
     def get_tag_by_id(self, tag_id):
         """Get a Tag object from a tag id."""
-        return self.sa_session.query(galaxy.model.Tag).filter_by(id=tag_id).first()
+        return self.sa_session.get(galaxy.model.Tag, tag_id)
 
     def get_tag_by_name(self, tag_name):
         """Get a Tag object from a tag name (string)."""
         if tag_name:
-            return self.sa_session.query(galaxy.model.Tag).filter_by(name=tag_name.lower()).first()
+            return self.sa_session.scalars(select(galaxy.model.Tag).filter_by(name=tag_name.lower()).limit(1)).first()
         return None
 
     def _create_tag(self, tag_str: str):
@@ -244,7 +316,7 @@ class TagHandler:
         return tag
 
     def _get_tag(self, tag_name):
-        return self.sa_session.query(galaxy.model.Tag).filter_by(name=tag_name).first()
+        return self.sa_session.scalars(select(galaxy.model.Tag).filter_by(name=tag_name).limit(1)).first()
 
     def _create_tag_instance(self, tag_name):
         # For good performance caller should first check if there's already an appropriate tag
@@ -255,8 +327,8 @@ class TagHandler:
         with Session() as separate_session:
             separate_session.add(tag)
             try:
-                separate_session.commit()
-                separate_session.flush()
+                with transaction(separate_session):
+                    separate_session.commit()
             except IntegrityError:
                 # tag already exists, get from database
                 separate_session.rollback()
@@ -329,7 +401,7 @@ class TagHandler:
         if not value:
             return None
         # Remove whitespace from value.
-        reg_exp = re.compile(r'\s')
+        reg_exp = re.compile(r"\s")
         scrubbed_value = re.sub(reg_exp, "", value)
         return scrubbed_value
 
@@ -339,7 +411,7 @@ class TagHandler:
         if not name:
             return None
         # Remove whitespace from name.
-        reg_exp = re.compile(r'\s')
+        reg_exp = re.compile(r"\s")
         scrubbed_name = re.sub(reg_exp, "", name)
         # Ignore starting ':' char.
         if scrubbed_name.startswith(self.hierarchy_separator):
@@ -359,7 +431,7 @@ class TagHandler:
     def _get_name_value_pair(self, tag_str) -> List[Optional[str]]:
         """Get name, value pair from a tag string."""
         # Use regular expression to parse name, value.
-        if tag_str.startswith('#'):
+        if tag_str.startswith("#"):
             tag_str = f"name:{tag_str[1:]}"
         reg_exp = re.compile(f"[{self.key_value_separators}]")
         name_value_pair: List[Optional[str]] = list(reg_exp.split(tag_str, 1))
@@ -370,41 +442,58 @@ class TagHandler:
 
 
 class GalaxyTagHandler(TagHandler):
-    def __init__(self, sa_session: galaxy_scoped_session):
+    _item_tag_assoc_info: Dict[str, ItemTagAssocInfo] = {}
+
+    def __init__(self, sa_session: galaxy_scoped_session, galaxy_session=None):
+        TagHandler.__init__(self, sa_session, galaxy_session=galaxy_session)
+        if not GalaxyTagHandler._item_tag_assoc_info:
+            GalaxyTagHandler.init_tag_associations()
+        self.item_tag_assoc_info = GalaxyTagHandler._item_tag_assoc_info
+
+    @classmethod
+    def init_tag_associations(cls):
         from galaxy import model
-        TagHandler.__init__(self, sa_session)
-        self.item_tag_assoc_info["History"] = ItemTagAssocInfo(model.History,
-                                                               model.HistoryTagAssociation,
-                                                               model.HistoryTagAssociation.history_id)
-        self.item_tag_assoc_info["HistoryDatasetAssociation"] = \
-            ItemTagAssocInfo(model.HistoryDatasetAssociation,
-                             model.HistoryDatasetAssociationTagAssociation,
-                             model.HistoryDatasetAssociationTagAssociation.history_dataset_association_id)
-        self.item_tag_assoc_info["HistoryDatasetCollectionAssociation"] = \
-            ItemTagAssocInfo(model.HistoryDatasetCollectionAssociation,
-                             model.HistoryDatasetCollectionTagAssociation,
-                             model.HistoryDatasetCollectionTagAssociation.history_dataset_collection_id)
-        self.item_tag_assoc_info["LibraryDatasetDatasetAssociation"] = \
-            ItemTagAssocInfo(model.LibraryDatasetDatasetAssociation,
-                             model.LibraryDatasetDatasetAssociationTagAssociation,
-                             model.LibraryDatasetDatasetAssociationTagAssociation.library_dataset_dataset_association_id)
-        self.item_tag_assoc_info["Page"] = ItemTagAssocInfo(model.Page,
-                                                            model.PageTagAssociation,
-                                                            model.PageTagAssociation.page_id)
-        self.item_tag_assoc_info["StoredWorkflow"] = ItemTagAssocInfo(model.StoredWorkflow,
-                                                                      model.StoredWorkflowTagAssociation,
-                                                                      model.StoredWorkflowTagAssociation.stored_workflow_id)
-        self.item_tag_assoc_info["Visualization"] = ItemTagAssocInfo(model.Visualization,
-                                                                     model.VisualizationTagAssociation,
-                                                                     model.VisualizationTagAssociation.visualization_id)
+
+        cls._item_tag_assoc_info = {
+            "History": ItemTagAssocInfo(
+                model.History, model.HistoryTagAssociation, model.HistoryTagAssociation.history_id
+            ),
+            "HistoryDatasetAssociation": ItemTagAssocInfo(
+                model.HistoryDatasetAssociation,
+                model.HistoryDatasetAssociationTagAssociation,
+                model.HistoryDatasetAssociationTagAssociation.history_dataset_association_id,
+            ),
+            "HistoryDatasetCollectionAssociation": ItemTagAssocInfo(
+                model.HistoryDatasetCollectionAssociation,
+                model.HistoryDatasetCollectionTagAssociation,
+                model.HistoryDatasetCollectionTagAssociation.history_dataset_collection_id,
+            ),
+            "LibraryDatasetDatasetAssociation": ItemTagAssocInfo(
+                model.LibraryDatasetDatasetAssociation,
+                model.LibraryDatasetDatasetAssociationTagAssociation,
+                model.LibraryDatasetDatasetAssociationTagAssociation.library_dataset_dataset_association_id,
+            ),
+            "Page": ItemTagAssocInfo(model.Page, model.PageTagAssociation, model.PageTagAssociation.page_id),
+            "StoredWorkflow": ItemTagAssocInfo(
+                model.StoredWorkflow,
+                model.StoredWorkflowTagAssociation,
+                model.StoredWorkflowTagAssociation.stored_workflow_id,
+            ),
+            "Visualization": ItemTagAssocInfo(
+                model.Visualization,
+                model.VisualizationTagAssociation,
+                model.VisualizationTagAssociation.visualization_id,
+            ),
+        }
+        return cls._item_tag_assoc_info
 
 
 class GalaxyTagHandlerSession(GalaxyTagHandler):
     """Like GalaxyTagHandler, but avoids one flush per created tag."""
 
-    def __init__(self, sa_session):
-        super().__init__(sa_session)
-        self.created_tags = {}
+    def __init__(self, sa_session, galaxy_session: Optional["GalaxySession"]):
+        super().__init__(sa_session, galaxy_session)
+        self.created_tags: Dict[str, "Tag"] = {}
 
     def _get_tag(self, tag_name):
         """Get tag from cache or database."""
@@ -419,6 +508,9 @@ class GalaxyTagHandlerSession(GalaxyTagHandler):
 
 
 class GalaxySessionlessTagHandler(GalaxyTagHandlerSession):
+    def _ensure_user_owns_item(self, user: Optional["User"], item):
+        # In sessionless mode we don't need to check ownership, we're only exporting
+        pass
 
     def _get_tag(self, tag_name):
         """Get tag from cache or database."""
